package it.flavianopetrocchi.jpdfbookmarks;

import java.nio.charset.Charset;
import java.util.Map;
import javax.swing.JOptionPane;

public class GeneralOptionsPanel extends javax.swing.JPanel {

    Prefs userPrefs;
    int defCharsetIndex = 0;
    String defCharset = Charset.defaultCharset().displayName();
    String defCharsetDecorated = defCharset + " (" + Res.getString("SYSTEM_DEFAULT") + ")";

    /** Creates new form GeneralOptionsPanel */
    public GeneralOptionsPanel(Prefs userPrefs) {
        this.userPrefs = userPrefs;
        initComponents();

        checkConvertNamedDestinations.setSelected(userPrefs.getConvertNamedDestinations());
        checkUseThousandthsUnits.setSelected(userPrefs.getUseThousandths());
        checkUseThousandthsUnits.setVisible(false);
        
        if (userPrefs.getNumClicks() > 1) {
            checkUseDoubleClicks.setSelected(true);
        } else {
            checkUseSingleClicks.setSelected(true);
        }

        Map<String, Charset> charsets = Charset.availableCharsets();
        for (String csName : charsets.keySet()) {
            if (csName.equalsIgnoreCase(defCharset)) {
                charsetsCombo.addItem(defCharsetDecorated);
            } else {
                charsetsCombo.addItem(csName);
            }
        }
        String userCharset = userPrefs.getCharsetEncoding();
        if (userCharset.equalsIgnoreCase(defCharset)) {
            charsetsCombo.setSelectedItem(defCharsetDecorated);
        } else {
            charsetsCombo.setSelectedItem(userCharset);
        }
    }

    public String getCharsetEncoding() {
        String selection = (String) charsetsCombo.getSelectedItem();
        if (selection.equalsIgnoreCase(defCharsetDecorated)) {
            return defCharset;
        }
        return selection;
    }

    public boolean convertNamedDestinations() {
        return checkConvertNamedDestinations.isSelected();
    }

    public boolean useThousandths() {
        return checkUseThousandthsUnits.isSelected();
    }
    
    public int getNumClicks() {
        if (checkUseSingleClicks.isSelected()) {
            return 1;
        } else {
            return 2;
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        groupClicks = new javax.swing.ButtonGroup();
        jPanel1 = new javax.swing.JPanel();
        checkConvertNamedDestinations = new javax.swing.JCheckBox();
        checkUseThousandthsUnits = new javax.swing.JCheckBox();
        charsetsCombo = new javax.swing.JComboBox();
        jLabel7 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        checkUseSingleClicks = new javax.swing.JRadioButton();
        checkUseDoubleClicks = new javax.swing.JRadioButton();

        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("it/flavianopetrocchi/jpdfbookmarks/locales/localizedText"); // NOI18N
        checkConvertNamedDestinations.setText(bundle.getString("CHECK_CONVERT_NAMED_DEST")); // NOI18N
        checkConvertNamedDestinations.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkConvertNamedDestinationsActionPerformed(evt);
            }
        });

        checkUseThousandthsUnits.setText(bundle.getString("CHECK_USE_THOUSANDTHS")); // NOI18N
        checkUseThousandthsUnits.setEnabled(false);

        jLabel7.setText(bundle.getString("SELECT_ENCODING")); // NOI18N

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder(bundle.getString("CHOOSE_NUM_CLICKS"))); // NOI18N

        groupClicks.add(checkUseSingleClicks);
        checkUseSingleClicks.setText(bundle.getString("SINGLE_CLICK")); // NOI18N

        groupClicks.add(checkUseDoubleClicks);
        checkUseDoubleClicks.setText(bundle.getString("DOUBLE_CLICK")); // NOI18N

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(checkUseSingleClicks)
                .addGap(18, 18, 18)
                .addComponent(checkUseDoubleClicks)
                .addContainerGap(388, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(checkUseSingleClicks)
                    .addComponent(checkUseDoubleClicks))
                .addContainerGap())
        );

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(20, 20, 20)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel2, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(checkConvertNamedDestinations)
                        .addGap(14, 14, 14)
                        .addComponent(checkUseThousandthsUnits))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel7)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(charsetsCombo, 0, 357, Short.MAX_VALUE)))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(checkConvertNamedDestinations)
                    .addComponent(checkUseThousandthsUnits))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel7)
                    .addComponent(charsetsCombo, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 634, Short.MAX_VALUE)
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(layout.createSequentialGroup()
                    .addContainerGap()
                    .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 172, Short.MAX_VALUE)
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(layout.createSequentialGroup()
                    .addContainerGap()
                    .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void checkConvertNamedDestinationsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_checkConvertNamedDestinationsActionPerformed
        JOptionPane.showMessageDialog(this,
                Res.getString("CONVERT_NAMED_DEST_MSG"), JPdfBookmarks.APP_NAME,
                JOptionPane.WARNING_MESSAGE);
}//GEN-LAST:event_checkConvertNamedDestinationsActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox charsetsCombo;
    private javax.swing.JCheckBox checkConvertNamedDestinations;
    private javax.swing.JRadioButton checkUseDoubleClicks;
    private javax.swing.JRadioButton checkUseSingleClicks;
    private javax.swing.JCheckBox checkUseThousandthsUnits;
    private javax.swing.ButtonGroup groupClicks;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    // End of variables declaration//GEN-END:variables
}
