/*
 * OptionsDialog.java
 *
 * Copyright (c) 2010 Flaviano Petrocchi <flavianopetrocchi at gmail.com>.
 * All rights reserved.
 *
 * This file is part of JPdfBookmarks.
 *
 * JPdfBookmarks is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JPdfBookmarks is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JPdfBookmarks.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.flavianopetrocchi.jpdfbookmarks;

import java.awt.BorderLayout;
import javax.swing.JOptionPane;

public class OptionsDlg extends javax.swing.JDialog {
    public final static int GENERAL_PANEL = 0;
    public final static int PROXY_PANEL = 1;
    public final static int TOOLBARS_PANEL = 2;

    private Prefs userPrefs = new Prefs();
    private SeparatorsPanel separatorsOptions = new SeparatorsPanel(userPrefs);
    private ConnectionOptionsPanel connectionOptions = new ConnectionOptionsPanel(userPrefs);
    private ToolbarsOptionsPanel toolbarsOptions = new ToolbarsOptionsPanel(userPrefs);
    private GeneralOptionsPanel generalOptions = new GeneralOptionsPanel(userPrefs);
//    private EncodingOptionsPanel encodingOptions = new EncodingOptionsPanel(userPrefs);
    private JPdfBookmarksGui gui;


    /** Creates new form OptionsDlg */
    public OptionsDlg(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        gui = (JPdfBookmarksGui) parent;
        mainTabPane.addTab(Res.getString("TAB_GENERAL_OPTIONS"), generalOptions);
        mainTabPane.setMnemonicAt(0, Res.mnemonicFromRes("TAB_GENERAL_OPTIONS_MNEMONIC"));
        mainTabPane.addTab(Res.getString("TAB_SEPARATOR_OPTIONS"), separatorsOptions);
        mainTabPane.setMnemonicAt(0, Res.mnemonicFromRes("TAB_SEPARATOR_OPTIONS_MNEMONIC"));
        mainTabPane.addTab(Res.getString("TAB_CONNECTION_OPTIONS"), connectionOptions);
        mainTabPane.setMnemonicAt(0, Res.mnemonicFromRes("TAB_CONNECTION_OPTIONS_MNEMONIC"));
        mainTabPane.add(Res.getString("TAB_TOOLBARS_MANAGER"), toolbarsOptions);
//        mainTabPane.setMnemonicAt(0, Res.mnemonicFromRes("TAB_ENCODING_OPTIONS_MNEMONIC"));
//        mainTabPane.add(Res.getString("TAB_ENCODING_OPTIONS"), encodingOptions);
        
        pack();
    }

    public void setVisibleTab(int tab) {
        mainTabPane.setSelectedIndex(tab);
    }

    private boolean checkDifferentValuesAndEmptyValues() {
        String pag = separatorsOptions.getPageSeparator();
        String ind = separatorsOptions.getIndentationString();
        String sep = separatorsOptions.getAttributesSeparator();


        if (pag.equals(ind) || pag.equals(sep) || ind.equals(sep)) {
            JOptionPane.showMessageDialog(this,
                    Res.getString("ERROR_DUPLICATED_VALUES"),
                    JPdfBookmarks.APP_NAME, JOptionPane.WARNING_MESSAGE);
            return false;
        }

        if (pag.isEmpty() || sep.isEmpty() || ind.isEmpty()) {
            JOptionPane.showMessageDialog(this,
                    Res.getString("ERROR_EMPTY_FIELD"),
                    JPdfBookmarks.APP_NAME, JOptionPane.WARNING_MESSAGE);
            return false;
        }

        return true;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        mainPanel = new javax.swing.JPanel();
        buttonsPanel = new javax.swing.JPanel();
        btnCancel = new javax.swing.JButton();
        btnOk = new javax.swing.JButton();
        tabsPanel = new javax.swing.JPanel();
        mainTabPane = new javax.swing.JTabbedPane();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("JPdfBookmarks"); // NOI18N

        mainPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        mainPanel.setLayout(new java.awt.BorderLayout());

        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("it/flavianopetrocchi/jpdfbookmarks/locales/localizedText"); // NOI18N
        btnCancel.setText(bundle.getString("CANCEL")); // NOI18N
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        btnOk.setText(bundle.getString("OK")); // NOI18N
        btnOk.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOkActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout buttonsPanelLayout = new javax.swing.GroupLayout(buttonsPanel);
        buttonsPanel.setLayout(buttonsPanelLayout);
        buttonsPanelLayout.setHorizontalGroup(
            buttonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(buttonsPanelLayout.createSequentialGroup()
                .addGap(475, 475, 475)
                .addComponent(btnOk)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnCancel)
                .addContainerGap())
        );

        buttonsPanelLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {btnCancel, btnOk});

        buttonsPanelLayout.setVerticalGroup(
            buttonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(buttonsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(buttonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnCancel)
                    .addComponent(btnOk))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        mainPanel.add(buttonsPanel, java.awt.BorderLayout.PAGE_END);

        tabsPanel.setLayout(new java.awt.BorderLayout());
        tabsPanel.add(mainTabPane, java.awt.BorderLayout.CENTER);

        mainPanel.add(tabsPanel, java.awt.BorderLayout.CENTER);

        jScrollPane1.setViewportView(mainPanel);

        getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void btnOkActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnOkActionPerformed

            if (!checkDifferentValuesAndEmptyValues()) {
                return;
            }

            String pag = separatorsOptions.getPageSeparator();
            String ind = separatorsOptions.getIndentationString();
            String sep = separatorsOptions.getAttributesSeparator();

            userPrefs.setPageSeparator(pag);
            userPrefs.setIndentationString(ind);
            userPrefs.setAttributesSeparator(sep);
            userPrefs.setConvertNamedDestinations(generalOptions.convertNamedDestinations());
            userPrefs.setUseThousandths(generalOptions.useThousandths());
            userPrefs.setCharsetEncoding(generalOptions.getCharsetEncoding());
            userPrefs.setNumClicks(generalOptions.getNumClicks());

            userPrefs.setUseProxy(connectionOptions.useProxy());
            userPrefs.setProxyType(connectionOptions.getProxyType());
            userPrefs.setProxyAddress(connectionOptions.getProxyAddress());
            userPrefs.setProxyPort(connectionOptions.getProxyPort());

            userPrefs.setCheckUpdatesOnStart(connectionOptions.checkUpdatesOnStart());

            userPrefs.setNeverAskWebAccess(connectionOptions.neverAskWebAccess());

            toolbarsOptions.saveToolbarPreferences();
            gui.updateToolbars();
            
            dispose();
	}//GEN-LAST:event_btnOkActionPerformed

	private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
            dispose();
	}//GEN-LAST:event_btnCancelActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnOk;
    private javax.swing.JPanel buttonsPanel;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JTabbedPane mainTabPane;
    private javax.swing.JPanel tabsPanel;
    // End of variables declaration//GEN-END:variables
}
